@@ -401,7 +401,7 @@ print.sf = function(x, ..., n = getOption("sf_max_print", default = 10)) {
 	nf = length(x) - length(geoms)
 	app = paste("and", nf, ifelse(nf == 1, "field", "fields"))
 	if (any(!is.na(st_agr(x))))
-		app = paste0(app, "\n", "Attribute-geometry relationship: ", summarize_agr(x))
+		app = paste0(app, "\n", "Attribute-geometry relationship", ifelse(nf > 1, "s: ", ": "), summarize_agr(x))
 	if (length(geoms) > 1)
 		app = paste0(app, "\n", "Active geometry column: ", attr(x, "sf_column"))
 	print(st_geometry(x), n = 0, what = "Simple feature collection with", append = app)
@@ -450,6 +450,23 @@ as.data.frame.sf = function(x, ...) {
 	NextMethod()
 }
 
+#' @export
+duplicated.sf <- function(x, incomparables = FALSE, fromLast = FALSE, ...) {
+  if (length(x) != 1L) {
+    if (any(i <- vapply(x, is.factor, NA))) {
+      for (j in names(i[i])) {
+        x[[j]] <- lapply(x[[j]], as.numeric)
+      }
+    }
+    if (any(i <- (lengths(lapply(x, dim)) == 2L))) {
+      for (j in names(i[i])) {
+        x[[j]] <- lapply(x[[j]], split.data.frame, seq_len(nrow(x)))
+      }
+    }
+  }
+  NextMethod()
+}
+
 #' @export
 #' @name st_geometry
 #' @details if \code{x} is of class \code{sf}, \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly; otherwise it returns \code{x} unmodified. 
