@@ -159,6 +159,7 @@ st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE, d
 #' @export
 c.sfc = function(..., recursive = FALSE) {
 	lst = list(...)
+	chk_equal_crs(lst)
 	classes = sapply(lst, function(x) class(x)[1])
 	le = lengths(lst)
 	if (any(le > 0))
@@ -218,16 +219,15 @@ print.sfc = function(x, ..., n = 5L, what = "Geometry set for", append = "") {
 	else {
 		p = crs_parameters(crs)
 		if (p$Name == "unknown") {
-			if (!is.character(crs$input) || is.na(crs$input))
-				cat(paste0("proj4string:   ", crs$proj4string, "\n"))
+			if (is.character(crs$input) && !is.na(crs$input) && crs$input != "unknown")
+				p$Name = crs$input
 			else
-				cat(paste0("CRS:           ", crs$input, "\n"))
-		} else if (p$IsGeographic)
+				p$Name = crs$proj4string
+		}
+		if (p$IsGeographic)
 			cat(paste0("Geodetic CRS:  ", p$Name, "\n"))
 		else
 			cat(paste0("Projected CRS: ", p$Name, "\n"))
-#		if (!is.na(crs$epsg))
-#			cat(paste0("epsg (SRID):    ", crs$epsg, "\n"))
 	}
 	if (attr(x, "precision") != 0.0) {
 		cat(    paste0("Precision:     "))
@@ -429,8 +429,8 @@ st_set_precision.sfc <- function(x, precision) {
     }
 
 	if (inherits(precision, "units")) {
-		u = st_crs(x, parameters=TRUE)$ud_unit
-		if (is.null(u) || !inherits(u, "units"))
+		u = st_crs(x)$ud_unit
+		if (is.null(u))
 			stop("cannot use precision expressed as units when target object has no units (CRS) set")
 		units(precision) = 1/u # convert
 		precision = as.numeric(precision)
@@ -471,7 +471,14 @@ typed_empty = function(cls, ncol = 2, dim = "XY") {
 #' retrieve coordinates in matrix form
 #' @param x object of class sf, sfc or sfg
 #' @param ... ignored
-#' @return matrix with coordinates (X, Y, possibly Z and/or M) in rows, possibly followed by integer indicators \code{L1},...,\code{L3} that point out to which structure the coordinate belongs; for \code{POINT} this is absent (each coordinate is a feature), for \code{LINESTRING} \code{L1} refers to the feature, for \code{MULTIPOLYGON} \code{L1} refers to the main ring or holes, \code{L2} to the ring id in the \code{MULTIPOLYGON}, and \code{L3} to the simple feature.
+#' @return matrix with coordinates (X, Y, possibly Z and/or M) in rows, possibly followed by integer indicators \code{L1},...,\code{L3} that point out to which structure the coordinate belongs; for \code{POINT} this is absent (each coordinate is a feature), for \code{LINESTRING} \code{L1} refers to the feature, for \code{MULTILINESTRING} \code{L1} refers to the part and \code{L2} to the simple feature, for \code{POLYGON} \code{L1} refers to the main ring or holes and \code{L2} to the simple feature, for \code{MULTIPOLYGON} \code{L1} refers to the main ring or holes, \code{L2} to the ring id in the \code{MULTIPOLYGON}, and \code{L3} to the simple feature.
+#' 
+#' For \code{POLYGONS}, \code{L1} can be used to identify exterior rings and inner holes. 
+#' The exterior ring is when \code{L1} is equal to 1. Interior rings are identified 
+#' when \code{L1} is greater than 1. \code{L2} can be used to differentiate between the 
+#' feature. Whereas for \code{MULTIPOLYGON}, \code{L3} refers to the \code{MULTIPOLYGON}
+#' feature and \code{L2} refers to the component \code{POLYGON}.
+#' 
 #' @export
 st_coordinates = function(x, ...) UseMethod("st_coordinates")
 
@@ -488,7 +495,7 @@ st_coordinates.sfc = function(x, ...) {
 
 	ret = switch(class(x)[1],
 		sfc_POINT = matrix(unlist(x, use.names = FALSE), nrow = length(x), byrow = TRUE,
-		     dimnames = list(seq_along(x))),
+		     dimnames = NULL),
 		sfc_MULTIPOINT = ,
 		sfc_LINESTRING = coord_2(x),
 		sfc_MULTILINESTRING = ,
